import _ from 'underscore'
import queryString from 'query-string'
import cookie from '../managers/cookie'
import auth from '../managers/auth'
import axios from '../managers/axios'

class BaseApi {

    constructor() {
        this._METHOD_GET = 'get'
        this._METHOD_POST = 'post'
        this._METHOD_PATCH = 'patch'
        this._METHOD_DELETE = 'delete'
        // this.interceptor()
    }

    /**
     * Get api path
     * @return {string}
     */
    getApiPath() {
    }

    /** Assign an object */
    getDataObject() {
    }

    /**
     * Get record list
     * @param {object} params
     * @return {Promise}
     */
    getRecordsList(params) {
        let additionalParams = params ? '?' + queryString.stringify(params) : ''
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_GET, _this.getApiPath() + additionalParams)
                .then(result => {
                    let newArray = []
                    if (result.total !== 0) {
                        let iThis = _this
                        _.map(result.data.records, function (record) {
                            newArray.push(iThis.getDataObject().parseDataFromResourceServer(record))
                        })
                        result.data.records = newArray
                        resolve(result.data)
                    } else {
                        resolve(result)
                    }
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Get single record
     * @param {number} recordId
     * @return {Promise}
     */
    getSingleRecord(recordId) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_GET, _this.buildUrlForSingleRecord(recordId), [])
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Create record
     * @param {object} data
     * @return {Promise}
     */
    createRecord(data) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_POST, _this.getApiPath(), data)
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Update record
     * @param {number} recordId
     * @param {object} data
     * @return {Promise}
     */
    updateRecord(recordId, data) {
        let _this = this
        return new Promise(function (resolve, reject) {
            _this.makeApiRequest(_this._METHOD_PATCH, _this.buildUrlForSingleRecord(recordId), data)
                .then(result => {
                    resolve(_this.getDataObject().parseDataFromResourceServer(result.data))
                })
                .catch(error => {
                    reject(error)
                })
        })
    }

    /**
     * Delete record
     * @param {number} recordId
     * @return {Promise}
     */
    deleteRecord(recordId) {
        return this.makeApiRequest(this._METHOD_DELETE, this.buildUrlForSingleRecord(recordId), [])
    }

    /**
     * Build url for single record
     * @param {number} recordId
     * @return {string}
     */
    buildUrlForSingleRecord(recordId) {
        return this.getApiPath() + '/' + recordId
    }

    /**
     * Make api request
     * @param {string} method
     * @param {string} url
     * @param {array} data
     * @return {Promise}
     */
    makeApiRequest(method, url, data) {
        return this.callApi(method, 'api/' + url, data)
    }

    interceptor() {
        // request happnas multiple times
        axios.interceptors.response.use(undefined, function (err) {
            return new Promise(function (resolve, reject) {
                if (err.response.status === 401 && err.config && !err.config.__isRetryRequest) {
                    auth.refreshToken()
                        .then(response => {
                            cookie.setToken(response.data)
                            err.config.__isRetryRequest = true
                            err.config.headers.Authorization = 'Bearer ' + cookie.getCurrentToken()
                            // err.config.headers.Authorization = 'Bearer ' + response.data.access_token
                            axios(err.config).then(resolve, reject);
                        })
                        .catch(error => {
                            console.log('Refresh login error: ', error);
                            reject(error);
                        })
                }
                throw err;
            });
        });
        /*axios.interceptors.response.use(function (response) {
            console.log('response')
            console.log(response)
            // Do something with response data
            return response;
        }, function (error) {
            console.log('error')
            console.log(error)
            if (err.response.status === 401 && err.response.config && !err.response.config.__isRetryRequest) {
                // get url request from response or request
            }
            // Do something with response error

            // ?????
            // return Promise.resolve(refreshToken().then(_ => Vue.http(req)))
            return Promise.reject(error);
        });*/
    }

    /**
     * Call api
     * @param {string} method
     * @param {string} url
     * @param {array} data
     * @return {Promise}
     */
    callApi(method, url, data) {
        const config = {
            method,
            url,
            data,
            headers: {}
        }
        // config.headers['Authorization'] = 'Bearer YWYxZTYwYWRkMjE3YjFkOWE1N2UzYjZlNzc2MmJjNmRjMDJlMDAxYzBiMjA4N2VlMmJmYTg5MDkxYWNhMTIwMg'
        if (cookie.getCurrentToken()) {
            config.headers['Authorization'] = 'Bearer ' + cookie.getCurrentToken()
        }
        return axios.request(config)
    }
}

export default BaseApi
